----------------------------------------------------------------------------------
-- Gibbous' Aura Watch : Part of the GibsUI Package
-- Version : 1.0a
-- Author : Gibbous <Crimson Knights>
-- Contact : Rockman922@hotmail.com
-- Environment : RIFT patch 1.5

-- the purpose of this addon is to alert the player when they, or their target
-- are missing buffs or debuffs that they're concerned about.

-- please do contact in the event of bugs, or feature requests, either at the
-- above email, or on RiftUI.com
----------------------------------------------------------------------------------
-- CURRENT FEATURE LIST
-- > Tracks all buffs by name inserted and target selected
-- > Easy to use list-based GUI
-- > Persistent GUI position and visibility between sessions
-- > Simple screen alert when buffs are missing
-- > Right-click and drag to reposition UI
----------------------------------------------------------------------------------
-- PLANNED FEATURE LIST
-- > Limited list with a slider ( to cap the UI size )
-- > Performance boosts
-- > Clear All button
-- > A fancy 'open UI' button (similar to Minmax Meter)
-- > Moveable Alert Frame
-- > Target.target and Focus target
----------------------------------------------------------------------------------
-- > KNOWN BUGS
----------------------------------------------------------------------------------

-- globals
GibsAW = {
	trackedAuras = {}, -- format : trackedAuras["targetmode..auraName"] = targetmode (for easy substring)
	numTracked = 0,
	versionNumber = "1.0a",
	alertActive = false,
	selectedAuraKey = "none",
	timeSinceLastUpdate = 0.5,
	updateInterval = 0.5,
	startTime = 0,
	GUI = {
		x = 300,
		y = 300,
		width = 300,
		minHeight = 90,
		frames = {},
		deadFrames = {}, -- dead aura frame storage
		context = nil
	}
}

GibsAW.GUI.context = UI.CreateContext("GibsAW")
-- base frame
local BGFrame = UI.CreateFrame("Frame", "GibsAWBG", GibsAW.GUI.context)
BGFrame:SetWidth(GibsAW.GUI.width)
BGFrame:SetHeight(GibsAW.GUI.minHeight)
BGFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", GibsAW.GUI.x, GibsAW.GUI.y)
BGFrame:SetBackgroundColor(0,0,0,0.5)
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- SavedVariables....
GibsUI_AWGUI_Open = true
GibsUI_AWGUI_Position = {}
function savedVariablesLoad()
	GibsAW.GUI.frames["mainBG"]:SetVisible(GibsUI_AWGUI_Open)
	if GibsUI_AWGUI_Position.x == nil then
		GibsUI_AWGUI_Position.x = GibsAW.GUI.x
		GibsUI_AWGUI_Position.y = GibsAW.GUI.y
	end
	BGFrame:SetPoint("TOPLEFT", GibsAW.GUI.context, "TOPLEFT", GibsUI_AWGUI_Position.x, GibsUI_AWGUI_Position.y)
end
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- mousemove on base bar (thanks to Marru and Gumaden for the basecode)
function BGFrame.Event:RightDown()
		BGFrame.MouseDown = true
		mouseData = Inspect.Mouse()
		BGFrame.MyStartX = BGFrame:GetLeft()
		BGFrame.MyStartY = BGFrame:GetTop()
		BGFrame.StartX = mouseData.x - BGFrame.MyStartX
		BGFrame.StartY = mouseData.y - BGFrame.MyStartY
		tempX = BGFrame:GetLeft()
		tempY = BGFrame:GetTop()
		tempW = BGFrame:GetWidth()
		tempH = BGFrame:GetHeight()
		BGFrame:ClearAll()
		BGFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", tempX, tempY)
		BGFrame:SetWidth(tempW)
		BGFrame:SetHeight(tempH)
		BGFrame:SetBackgroundColor(0.2,0.2,0.6,1)
end
function BGFrame.Event.MouseMove()
		if BGFrame.MouseDown then
				local newX, newY
				mouseData = Inspect.Mouse()
				newX = mouseData.x - BGFrame.StartX
				newY = mouseData.y - BGFrame.StartY
				BGFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", newX, newY)
		end
end
function BGFrame.Event.RightUp()
	if BGFrame.MouseDown then
		GibsUI_AWGUI_Position.x = math.floor(BGFrame:GetLeft())
		GibsUI_AWGUI_Position.y = math.floor(BGFrame:GetTop())
		BGFrame.MouseDown = false
		BGFrame:SetBackgroundColor(0, 0, 0, 0.5)
	end
end
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- alert : string
-- returns : void
-- handles the alertbox function based on the aura that was passed
local function alert(aura)
	if not GibsAW.alertActive then -- if the alert is off
		GibsAW.alertActive = true -- turn it on
		GibsAW.GUI.frames["alertText"]:SetText(aura:upper().." IS MISSING") -- set its text
		GibsAW.GUI.frames["alertText"]:ResizeToText() -- resize
		GibsAW.GUI.frames["alertText"]:SetPoint("CENTER", UIParent, "CENTER", 0, -150) -- position
	end
end
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- search : unit
-- returns : void
-- searches the unit for auras associated with it, and alerts if necessary
local function search(unit)
	
	local unitTest = Inspect.Unit.Detail(unit) -- get the unit
	if unitTest and unitTest.health <= 0 then -- don't check for auras if the unit is dead and exists
		return
	end

	local balance -- translate player and player.target into the right unit text for parse
	if unit == "player" then
		balance = "self"
	else
		balance = "target"
	end
	
	-- Check appropriate targets for auras if they exist
	for k, v in pairs(GibsAW.trackedAuras) do -- k = target..aura, v = target
		if v == balance then -- if v and balance match (balance being the parse key)
		
			local check
			check = Inspect.Buff.List(unit) -- grab the unit's buff list
			if not check then -- if there's no buffs
				return -- nothing to se here
			end
			
			local checkDetails = Inspect.Buff.Detail(unit, check) -- get the unit's buff details
			local found = 0 -- local variable to track if a buff is present, 1 means yes
			for _, v2 in pairs(checkDetails) do -- v2 is the buff
				if v2.name == string.gsub(k, v, "") then -- if the name matches our watched aura
					found = 1 -- found it
					break -- kill the loop
				end
			end
			if found == 0 then -- if we didn't find the aura
				alert(string.gsub(k, v, "")) -- SOUND THE ALARM
			end
		end
	end
end
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- track : string, string
-- returns : void
-- handles tracking of buffs, basically just adds them to the hashes
local function track(aura, target)
	if GibsAW.trackedAuras[target..aura] == nil then -- if we haven't tracked it
		GibsAW.trackedAuras[target..aura] = target -- add it to the tracked auras
		GibsAW.numTracked = GibsAW.numTracked + 1 -- blood for the blood god
		
		local newTextBG = UI.CreateFrame("Frame", target..aura.."BGa", GibsAW.GUI.frames["listBG"]) -- each aura needs frames
		newTextBG:SetPoint("TOPCENTER", GibsAW.GUI.frames["listBG"], "TOPCENTER") -- background
		newTextBG:SetHeight(20)
		newTextBG:SetWidth(287)
		
		newTextBG.Event.MouseIn = function() -- function for mouse overs
			 -- on hover, highlight if it's not selected
			if GibsAW.selectedAuraKey == target..aura then
				return
			end
			newTextBG:SetBackgroundColor(0.2,0.2,0.6,1)
		end
		
		newTextBG.Event.MouseOut = function() -- function for mouse outs
			-- on move away, de-highlight if not selected
			if GibsAW.selectedAuraKey == target..aura then
				return
			end
			newTextBG:SetBackgroundColor(0,0,0,0) 
		end
		
		newTextBG.Event.LeftClick = function() -- function for clicking
			if GibsAW.selectedAuraKey == target..aura then -- if it's the current selected aura
				GibsAW.selectedAuraKey = "none" -- deselect it
				newTextBG:SetBackgroundColor(0,0,0,0) -- dehighlight
			else -- else
				if GibsAW.GUI.frames[GibsAW.selectedAuraKey.."BGa"] then
					GibsAW.GUI.frames[GibsAW.selectedAuraKey.."BGa"]:SetBackgroundColor(0,0,0,0) -- deselect the last one (if one)
				end
				GibsAW.selectedAuraKey = target..aura -- select it
 				newTextBG:SetBackgroundColor(0.6,0.6,0.2,1) -- highlight it
			end
		end
		
		local newTextText = UI.CreateFrame("Text", target..aura.."Text", newTextBG) -- need text too
 		newTextText:SetText(aura.." ("..target..")") -- example: Radiant Spores (self)
		newTextText:ResizeToText() 
		newTextText:SetPoint("TOPLEFT", newTextBG, "TOPLEFT", 2, 0) -- position
		
		GibsAW.GUI.frames[target..aura.."BGa"] = newTextBG -- store for future use
		GibsAW.GUI.frames[target..aura.."Text"] = newTextText -- ditto
	else
		print(aura.." ("..target..") is already being tracked") -- error message
	end
end
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- untrack : string
-- returns : void
-- removes an aura from the tracker by nil-ing it out
local function untrack(auraKey)
	GibsAW.trackedAuras[auraKey] = nil -- remove aura from the hash
	GibsAW.GUI.frames[auraKey.."BGa"]:SetVisible(false) -- set its frame to invisible
	GibsAW.numTracked = GibsAW.numTracked - 1 -- one less tracked
end
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- gui : void
-- returns : void
-- handles the GUI display code and states
local function gui()
	if GibsUI_AWGUI_Open then -- if the main GUI is open
		local size = GibsAW.numTracked -- grab the size of the table
		GibsAW.GUI.frames["listBG"]:SetHeight((20 * size)) -- dynamically size listBG
		GibsAW.GUI.frames["mainBG"]:SetHeight(90 + (20 * size)) -- dynamically size mainBG
		
		local index = 0 -- temp index (how many from the top?)
		for k,v in pairs(GibsAW.GUI.frames) do -- loop through all of the frames
			if string.find(k, "BGa") then -- if it's an aura list BG
				if GibsAW.trackedAuras[string.gsub(k, "BGa", "")] then -- if that aura is still tracked
					-- position it based on how many from the top it should be at this point
					GibsAW.GUI.frames[k]:SetPoint("TOPCENTER", GibsAW.GUI.frames["listBG"], "TOPCENTER", 0, (20 * (index))) 
					index = index + 1 -- skulls for the skull throne!
				end
			end
		end
	else
	end
	GibsAW.GUI.frames["mainBG"]:SetVisible(GibsUI_AWGUI_Open) -- keep the GUI state up to date
	if GibsAW.alertActive then -- if there's an alert
		GibsAW.GUI.frames["alertText"]:SetVisible(true) -- well damnit tell batman!
	else -- if there's no alert
		GibsAW.GUI.frames["alertText"]:SetVisible(false) -- don't bother him
	end
	GibsAW.alertActive = false -- preemptively dismiss alert
end
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- tick : void
-- returns : void
-- handles constant updates, processes only a few times a second
local function tick()
	if GibsAW.timeSinceLastUpdate >= GibsAW.updateInterval then -- if it's been GibsAW.updateInterval seconds
 		search("player") -- search the player
		search("player.target") -- search the target
		gui() -- update the GUI
		GibsAW.timeSinceLastUpdate = 0 -- it's been 0 seconds since update
		GibsAW.startTime = Inspect.Time.Frame() -- we updated at now
	else -- if it's not yet time to update
		GibsAW.timeSinceLastUpdate = Inspect.Time.Frame() - GibsAW.startTime -- increment timeSinceLastUpdate
	end
end
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- toggleUI : void
-- returns : void
-- toggles the GUI state
local function toggleUI()
	if GibsUI_AWGUI_Open then -- if the main GUI is open
 		GibsUI_AWGUI_Open = false -- close it
		GibsAW.GUI.frames["textbox"]:SetKeyFocus(false) -- defocus the text
	else -- otherwise
		GibsUI_AWGUI_Open = true -- open it
	end
end
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- help : void
-- returns : void
-- prints the help called upon by /awHelp
local function help()
	print("Welcome to GibsAW, version "..GibsAW.versionNumber)
	print("/awui : toggle main UI visibility")
	print("RIGHT CLICK and drag to move the UI frame")
end
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- init : void
-- returns : void
-- initializes all GUI frames, stores important ones, and registers events
local function init()
	GibsAW.startTime = Inspect.Time.Frame()
	
	-- initialize the GUI
	-- background frame
	
	--  create text field
	local textInput = UI.CreateFrame("RiftTextfield", "auraInput", BGFrame)
	textInput:SetPoint("TOPCENTER", BGFrame, "TOPCENTER", 0, 6)
	textInput:SetWidth(290)
	textInput:SetHeight(20)
	textInput:SetText("Type an aura name here")
	textInput:SetBackgroundColor(0,0,0,1)
	
	--  create self check
	local text = UI.CreateFrame("Text", "checkText", BGFrame)
	text:SetText("Self:        Target:        ")
	text:SetPoint("TOPLEFT", BGFrame, "TOPLEFT", 6, 32)
	text:ResizeToText()
	
	local selfCheck = UI.CreateFrame("RiftCheckbox", "selfCheck", BGFrame)
	selfCheck:SetPoint("TOPLEFT", BGFrame, "TOPLEFT", 35, 33)
	
	--  create target check
	local targetCheck = UI.CreateFrame("RiftCheckbox", "selfCheck", BGFrame)
	targetCheck:SetPoint("TOPLEFT", BGFrame, "TOPLEFT", 100, 33)
	
	selfCheck.Event.CheckboxChange = function()
		if selfCheck:GetChecked() then
			textInput:SetKeyFocus(false)
			targetCheck:SetChecked(false)
		end
	end
	
	targetCheck.Event.CheckboxChange = function()
		if targetCheck:GetChecked() then
			textInput:SetKeyFocus(false)
			selfCheck:SetChecked(false)
		end
	end
	
	--  create add frame
	local addBG = UI.CreateFrame("Frame", "addBG", BGFrame)
	addBG:SetPoint("TOPLEFT", BGFrame, "TOPLEFT", 130, 33)
	addBG:SetWidth(75)
	addBG:SetHeight(20)
	addBG:SetBackgroundColor(0,0,0,1)
	local add = UI.CreateFrame("Text", "addFrame", addBG)
	add:SetPoint("CENTER", addBG, "CENTER", 3, 11)
	add:SetText("ADD")
	
	addBG.Event.MouseIn = function()
		addBG:SetBackgroundColor(0.2,0.2,0.6,1)
	end
	
	addBG.Event.MouseOut = function()
		addBG:SetBackgroundColor(0,0,0,1)
	end
	
	addBG.Event.LeftClick = function()
		textInput:SetKeyFocus(false)
		if selfCheck:GetChecked() then
			track(textInput:GetText(), "self")
			textInput:SetText("Type an aura name here")
		elseif targetCheck:GetChecked() then
			track(textInput:GetText(), "target")
			textInput:SetText("Type an aura name here")
		else
			print("Select a unit checkbox")
		end
	end
	
	--  create remove frame
	local rBG = UI.CreateFrame("Frame", "rBG", BGFrame)
	rBG:SetPoint("TOPLEFT", BGFrame, "TOPLEFT", 220, 33)
	rBG:SetWidth(75)
	rBG:SetHeight(20)
	rBG:SetBackgroundColor(0,0,0,1)
	local rem = UI.CreateFrame("Text", "remFrame", rBG)
	rem:SetPoint("CENTER", rBG, "CENTER", -7, 11)
	rem:SetText("REMOVE")
	
	rBG.Event.MouseIn = function()
		rBG:SetBackgroundColor(0.2,0.2,0.6,1)
	end
	
	rBG.Event.MouseOut = function()
		rBG:SetBackgroundColor(0,0,0,1)
	end
	
	rBG.Event.LeftClick = function()
		if GibsAW.selectedAuraKey == "none" then
			print("Select an aura to remove")
			return
		end
		untrack(GibsAW.selectedAuraKey)
	end
	
	-- create close frame
	local closeBG = UI.CreateFrame("Frame", "closeBG", BGFrame)
	closeBG:SetPoint("BOTTOMLEFT", BGFrame, "BOTTOMLEFT", 6, -6)
	closeBG:SetWidth(288)
	closeBG:SetHeight(20)
	closeBG:SetBackgroundColor(0,0,0,1)
	local rem = UI.CreateFrame("Text", "closeFrame", closeBG)
	rem:SetPoint("CENTER", closeBG, "CENTER", -5, 11)
	rem:SetText("CLOSE")
	
	closeBG.Event.MouseIn = function() -- close hover over
		closeBG:SetBackgroundColor(0.2,0.2,0.6,1)
	end
	
	closeBG.Event.MouseOut = function() -- close hover out
		closeBG:SetBackgroundColor(0,0,0,1)
	end

	closeBG.Event.LeftClick = function() -- close left click
		toggleUI()
	end
	
	-- create list background
	local listBG = UI.CreateFrame("Frame", "listBG", BGFrame)
	listBG:SetPoint("TOPLEFT", BGFrame, "TOPLEFT", 6, 59)
	listBG:SetWidth(288)
	listBG:SetHeight(110)
	listBG:SetBackgroundColor(0,0,0,0.7)
	
	-- create the alert frame
	local alertText = UI.CreateFrame("Text", "alertTextGibsAW", GibsAW.GUI.context)
	alertText:SetText("PAY ATTENTION!")
	alertText:SetFontSize(25)
	alertText:ResizeToText()
	alertText:SetPoint("CENTER", UIParent, "CENTER", 0, -150)
	alertText:SetVisible(false) -- make it invisible
	
	-- most of them aren't going to change, so they're not needed
	GibsAW.GUI.frames["mainBG"] = BGFrame
	GibsAW.GUI.frames["listBG"] = listBG
	GibsAW.GUI.frames["alertText"] = alertText
	GibsAW.GUI.frames["textbox"] = textInput
	
	table.insert(Command.Slash.Register("gibsAW"), {help, "GibsAW", "help command"})
	table.insert(Command.Slash.Register("awui"), {toggleUI, "GibsAW", "ui toggle"})
	
	table.insert(Event.System.Update.Begin, {tick, "GibsAW", "update ticker"})
	table.insert(Event.Addon.SavedVariables.Load.End, {savedVariablesLoad, "GibsAW", "save the saved stuff"})
	
	print("Type /gibsAW for a list of commands and features")
end
----------------------------------------------------------------------------------

init() -- for great justice